@@ -1,7 +1,6 @@
 // SPDX-License-Identifier: LGPL-2.1-or-later
 // Copyright (c) 2012-2014 Monty Program Ab
-// Copyright (c) 2015-2021 MariaDB Corporation Ab
-
+// Copyright (c) 2015-2024 MariaDB Corporation Ab
 package org.mariadb.jdbc.integration;
 
 import static org.junit.jupiter.api.Assertions.*;
@@ -237,7 +236,9 @@ public class MultiHostTest extends Common {
   @Test
   public void masterFailover() throws Exception {
     Assumptions.assumeTrue(
-        !"skysql".equals(System.getenv("srv")) && !"skysql-ha".equals(System.getenv("srv")));
+        !"maxscale".equals(System.getenv("srv"))
+            && !"skysql".equals(System.getenv("srv"))
+            && !"skysql-ha".equals(System.getenv("srv")));
 
     Configuration conf = Configuration.parse(mDefUrl);
     HostAddress hostAddress = conf.addresses().get(0);
@@ -249,10 +250,13 @@ public class MultiHostTest extends Common {
 
     String url =
         mDefUrl.replaceAll(
-            "//([^/]*)/",
+            "//([^/]*)/" + database,
             String.format(
-                "//address=(host=localhost)(port=9999)(type=master),address=(host=localhost)(port=%s)(type=master),address=(host=%s)(port=%s)(type=master)/",
-                proxy.getLocalPort(), hostAddress.host, hostAddress.port));
+                "//address=(host=localhost)(port=9999)(type=master),address=(host=localhost)(port=%s)(type=master),address=(host=%s)(port=%s)(type=master)/"
+                    + database,
+                proxy.getLocalPort(),
+                hostAddress.host,
+                hostAddress.port));
     url = url.replaceAll("jdbc:mariadb:", "jdbc:mariadb:sequential:");
     if (conf.sslMode() == SslMode.VERIFY_FULL) {
       url = url.replaceAll("sslMode=verify-full", "sslMode=verify-ca");
@@ -269,18 +273,18 @@ public class MultiHostTest extends Common {
       con.isValid(1000);
     }
 
-    Thread.sleep(100);
+    Thread.sleep(1000);
     // same in transaction
     try (Connection con =
         (Connection)
             DriverManager.getConnection(
                 url
-                    + "&waitReconnectTimeout=300&retriesAllDown=10&connectTimeout=20&deniedListTimeout=20")) {
+                    + "&waitReconnectTimeout=300&retriesAllDown=10&connectTimeout=20&deniedListTimeout=20&socketTimeout=100")) {
       Statement stmt = con.createStatement();
       stmt.execute("START TRANSACTION");
       stmt.execute("SET @con=1");
 
-      proxy.restart(100);
+      proxy.restart(100, true);
       try {
         ResultSet rs = stmt.executeQuery("SELECT @con");
         if (rs.next()) {
@@ -312,7 +316,7 @@ public class MultiHostTest extends Common {
         assertTrue(e.getMessage().contains("In progress transaction was lost"));
       }
     }
-    Thread.sleep(50);
+    Thread.sleep(100);
     // with transaction replay
     try (Connection con =
         (Connection)
