@@ -1,6 +1,6 @@
 // SPDX-License-Identifier: LGPL-2.1-or-later
 // Copyright (c) 2012-2014 Monty Program Ab
-// Copyright (c) 2015-2023 MariaDB Corporation Ab
+// Copyright (c) 2015-2024 MariaDB Corporation Ab
 package org.mariadb.jdbc.integration;
 
 import static org.junit.jupiter.api.Assertions.*;
@@ -71,6 +71,83 @@ public class StatementTest extends Common {
     stmt.execute("DROP TABLE key_test");
   }
 
+  @Test
+  public void ensureJdbcErrorWhenNoResultset() throws SQLException {
+    Statement stmt = sharedConn.createStatement();
+    stmt.execute("DO 1");
+    assertThrowsContains(
+        SQLException.class,
+        () -> stmt.executeQuery("DO 1"),
+        "Statement.executeQuery() command does NOT return a result-set as expected. Either use"
+            + " Statement.execute(), Statement.executeUpdate(), or correct command");
+    stmt.execute("DO 1");
+    try (PreparedStatement ps =
+        sharedConn.prepareStatement("DO ?", Statement.RETURN_GENERATED_KEYS)) {
+      ps.setInt(1, 1);
+      ps.execute();
+      assertThrowsContains(
+          SQLException.class,
+          () -> ps.executeQuery(),
+          "PrepareStatement.executeQuery() command does NOT return a result-set as expected. Either"
+              + " use PrepareStatement.execute(), PrepareStatement.executeUpdate(), or correct"
+              + " command");
+      ps.execute();
+    }
+    try (Connection con = createCon("&permitNoResults=true")) {
+      try (PreparedStatement ps = con.prepareStatement("DO ?", Statement.RETURN_GENERATED_KEYS)) {
+        ps.setInt(1, 1);
+        ps.execute();
+        ResultSet rs = ps.executeQuery();
+        assertFalse(rs.next());
+        ps.execute();
+      }
+    }
+    try (PreparedStatement ps =
+        sharedConnBinary.prepareStatement("DO ?", Statement.RETURN_GENERATED_KEYS)) {
+      ps.setInt(1, 1);
+      ps.execute();
+      assertThrowsContains(
+          SQLException.class,
+          () -> ps.executeQuery(),
+          "PrepareStatement.executeQuery() command does NOT return a result-set as expected. Either"
+              + " use PrepareStatement.execute(), PrepareStatement.executeUpdate(), or correct"
+              + " command");
+      ps.execute();
+    }
+    try (Connection con = createCon("permitNoResults=true&useServerPrepStmts=true")) {
+      try (PreparedStatement ps = con.prepareStatement("DO ?", Statement.RETURN_GENERATED_KEYS)) {
+        ps.setInt(1, 1);
+        ps.execute();
+        ResultSet rs = ps.executeQuery();
+        assertFalse(rs.next());
+      }
+    }
+  }
+
+  @Test
+  public void getUpdateCountValueOnFail() throws SQLException {
+    try (Statement st = sharedConn.createStatement()) {
+      st.execute("DROP TABLE IF EXISTS getUpdateCountValueOnFail");
+      try (Statement stmt = sharedConn.createStatement()) {
+        assertEquals(-1, stmt.getUpdateCount());
+        assertEquals(
+            0,
+            stmt.executeUpdate(
+                "CREATE TABLE getUpdateCountValueOnFail(id VARCHAR(5) PRIMARY KEY,value BOOL)"));
+        assertEquals(0, stmt.getUpdateCount());
+        try {
+          stmt.executeUpdate(
+              "CREATE TABLE getUpdateCountValueOnFail(id TINYINT PRIMARY KEY,value SMALLINT");
+        } catch (Exception e) {
+          // eat
+        }
+        assertEquals(-1, stmt.getUpdateCount());
+      } finally {
+        st.execute("DROP TABLE IF EXISTS getUpdateCountValueOnFail");
+      }
+    }
+  }
+
   @Test
   public void longGeneratedId() throws SQLException {
     longGeneratedId(BigInteger.ONE);
@@ -222,6 +299,7 @@ public class StatementTest extends Common {
   @Test
   public void executeGenerated() throws SQLException {
     Statement stmt = sharedConn.createStatement();
+    stmt.execute("TRUNCATE TABLE executeGenerated");
     assertFalse(stmt.execute("INSERT INTO executeGenerated(t2) values (100)"));
 
     SQLException e = Assertions.assertThrows(SQLException.class, stmt::getGeneratedKeys);
@@ -235,6 +313,131 @@ public class StatementTest extends Common {
     assertEquals(2, rs.getInt(1));
   }
 
+  @Test
+  public void executeGeneratedMultiValues() throws SQLException {
+    // normal
+    Statement stmt = sharedConn.createStatement();
+    stmt.execute("TRUNCATE TABLE executeGenerated");
+    assertFalse(
+        stmt.execute(
+            "INSERT INTO executeGenerated(t2) values (100), (101)",
+            Statement.RETURN_GENERATED_KEYS));
+    ResultSet rs = stmt.getGeneratedKeys();
+    assertTrue(rs.next());
+    assertEquals(1, rs.getInt(1));
+    assertFalse(rs.next());
+
+    try (PreparedStatement prep =
+        sharedConn.prepareStatement(
+            "INSERT INTO executeGenerated(t2) values (?), (?)", Statement.RETURN_GENERATED_KEYS)) {
+      prep.setInt(1, 104);
+      prep.setInt(2, 105);
+      prep.execute();
+      rs = prep.getGeneratedKeys();
+      assertTrue(rs.next());
+      assertEquals(3, rs.getInt(1));
+      assertFalse(rs.next());
+    }
+
+    // with returnMultiValuesGeneratedIds options
+    stmt.execute("TRUNCATE TABLE executeGenerated");
+    try (Connection conn = createCon("&returnMultiValuesGeneratedIds")) {
+      Statement stmt2 = conn.createStatement();
+      assertFalse(
+          stmt2.execute(
+              "INSERT INTO executeGenerated(t2) values (102), (103)",
+              Statement.RETURN_GENERATED_KEYS));
+      rs = stmt2.getGeneratedKeys();
+      assertTrue(rs.next());
+      assertEquals(1, rs.getInt(1));
+      assertTrue(rs.next());
+      assertEquals(2, rs.getInt(1));
+      assertFalse(rs.next());
+      stmt.execute("TRUNCATE TABLE executeGenerated");
+
+      try (PreparedStatement prep =
+          conn.prepareStatement(
+              "INSERT INTO executeGenerated(t2) values (?), (?)",
+              Statement.RETURN_GENERATED_KEYS)) {
+        stmt2.execute("SET auto_increment_increment = 3");
+        prep.setInt(1, 104);
+        prep.setInt(2, 105);
+        prep.execute();
+        rs = prep.getGeneratedKeys();
+        assertTrue(rs.next());
+        assertEquals(1, rs.getInt(1));
+        assertTrue(rs.next());
+        assertEquals(4, rs.getInt(1));
+        assertFalse(rs.next());
+        ResultSet rs2 = stmt2.executeQuery("SELECT * FROM executeGenerated");
+        assertTrue(rs2.next());
+        assertEquals(1, rs2.getInt(1));
+        assertTrue(rs2.next());
+        assertEquals(4, rs2.getInt(1));
+        assertFalse(rs2.next());
+      }
+      stmt.execute("TRUNCATE TABLE executeGenerated");
+      try (PreparedStatement prep =
+          conn.prepareStatement(
+              "INSERT INTO executeGenerated(t2) values (?), (?) ON DUPLICATE KEY UPDATE"
+                  + " t2=CONCAT(t2,'a')",
+              Statement.RETURN_GENERATED_KEYS)) {
+        prep.setInt(1, 106);
+        prep.setInt(2, 107);
+        prep.execute();
+        rs = prep.getGeneratedKeys();
+        assertTrue(rs.next());
+        assertEquals(1, rs.getInt(1));
+        assertFalse(rs.next());
+      }
+    }
+
+    // with returnMultiValuesGeneratedIds options
+    stmt.execute("TRUNCATE TABLE executeGenerated");
+    try (Connection conn = createCon("&returnMultiValuesGeneratedIds&useServerPrepStmts")) {
+      Statement stmt2 = conn.createStatement();
+      assertFalse(
+          stmt2.execute(
+              "INSERT INTO executeGenerated(t2) values (102), (103)",
+              Statement.RETURN_GENERATED_KEYS));
+      rs = stmt2.getGeneratedKeys();
+      assertTrue(rs.next());
+      assertEquals(1, rs.getInt(1));
+      assertTrue(rs.next());
+      assertEquals(2, rs.getInt(1));
+      assertFalse(rs.next());
+      stmt.execute("TRUNCATE TABLE executeGenerated");
+      try (PreparedStatement prep =
+          conn.prepareStatement(
+              "INSERT INTO executeGenerated(t2) values (?), (?)",
+              Statement.RETURN_GENERATED_KEYS)) {
+        prep.setInt(1, 104);
+        prep.setInt(2, 105);
+        prep.execute();
+        rs = prep.getGeneratedKeys();
+        assertTrue(rs.next());
+        assertEquals(1, rs.getInt(1));
+        assertTrue(rs.next());
+        assertEquals(2, rs.getInt(1));
+        assertFalse(rs.next());
+      }
+      stmt.execute("TRUNCATE TABLE executeGenerated");
+      try (PreparedStatement prep =
+          conn.prepareStatement(
+              "INSERT INTO executeGenerated(t2) values (?), (?) ON DUPLICATE KEY UPDATE"
+                  + " t2=CONCAT(t2,'a')",
+              Statement.RETURN_GENERATED_KEYS)) {
+        prep.setInt(1, 106);
+        prep.setInt(2, 107);
+        prep.execute();
+        rs = prep.getGeneratedKeys();
+        assertTrue(rs.next());
+        assertEquals(1, rs.getInt(1));
+        assertFalse(rs.next());
+      }
+    }
+  }
+
   @Test
   public void executeGeneratedBatch() throws SQLException {
     Statement stmt = sharedConn.createStatement();
@@ -321,13 +524,13 @@ public class StatementTest extends Common {
   }
 
   @Test
-  public void executeQuery() throws SQLException {
+  public void executeUpdateNoResults() throws SQLException {
     Statement stmt = sharedConn.createStatement();
     ResultSet rs = stmt.executeQuery("SELECT 1");
     assertTrue(rs.next());
     if (!isXpand()) {
-      rs = stmt.executeQuery("DO 1");
-      assertFalse(rs.next());
+      stmt.executeUpdate("DO 1");
+      assertNull(stmt.getResultSet());
     }
   }
 
@@ -445,6 +648,19 @@ public class StatementTest extends Common {
       assertEquals(i, rs.getInt(1));
     }
     assertEquals(10, i);
+
+    try (Connection conn = createCon("&canUseServerTimeout=false")) {
+      Statement stmt2 = conn.createStatement();
+      stmt2.setQueryTimeout(2);
+      stmt2.setMaxRows(10);
+      rs = stmt2.executeQuery("SELECT * FROM sequence_1_to_10000");
+      i = 0;
+      while (rs.next()) {
+        i++;
+        assertEquals(i, rs.getInt(1));
+      }
+      assertEquals(10, i);
+    }
   }
 
   @Test
@@ -582,13 +798,7 @@ public class StatementTest extends Common {
 
   @Test
   @Timeout(20)
-  public void queryTimeout() {
-    Assumptions.assumeTrue(
-        isMariaDBServer()
-            && !"maxscale".equals(System.getenv("srv"))
-            && !"skysql".equals(System.getenv("srv"))
-            && !"skysql-ha".equals(System.getenv("srv"))
-            && !isXpand());
+  public void queryTimeout() throws SQLException {
     Statement stmt = sharedConn.createStatement();
 
     Common.assertThrowsContains(
@@ -603,7 +813,19 @@ public class StatementTest extends Common {
               "select * from information_schema.columns as c1,  information_schema.tables,"
                   + " information_schema.tables as t2");
         },
-        "Query execution was interrupted (max_statement_time exceeded)");
+        "Query execution was interrupted");
+    stmt.setQueryTimeout(1);
+    stmt.execute("SELECT 1");
+    Common.assertThrowsContains(
+        SQLTimeoutException.class,
+        () -> {
+          stmt.setQueryTimeout(1);
+          assertEquals(1, stmt.getQueryTimeout());
+          stmt.execute(
+              "select * from information_schema.columns as c1,  information_schema.tables,"
+                  + " information_schema.tables as t2");
+        },
+        "Query execution was interrupted");
   }
 
   @Test
@@ -1098,9 +1320,6 @@ public class StatementTest extends Common {
       prep.setInt(1, 5);
       prep.setString(2, "t55");
       prep.addBatch();
-      prep.setInt(1, 6);
-      prep.setString(2, "t6");
-      prep.addBatch();
       prep.setNull(1, Types.INTEGER);
       prep.setString(2, "t7");
       prep.addBatch();
@@ -1109,8 +1328,6 @@ public class StatementTest extends Common {
       rs = prep.getGeneratedKeys();
       assertTrue(rs.next());
       assertEquals(6, rs.getInt(1));
-      assertTrue(rs.next());
-      assertEquals(7, rs.getInt(1));
       assertFalse(rs.next());
     }
 
@@ -1121,7 +1338,7 @@ public class StatementTest extends Common {
       prep.setInt(1, 5);
       prep.setString(2, "t55");
       prep.addBatch();
-      prep.setInt(1, 8);
+      prep.setNull(1, Types.INTEGER);
       prep.setString(2, "t8");
       prep.addBatch();
       prep.setNull(1, Types.INTEGER);
@@ -1131,9 +1348,9 @@ public class StatementTest extends Common {
 
       rs = prep.getGeneratedKeys();
       assertTrue(rs.next());
-      assertEquals(8, rs.getInt(1));
+      assertEquals(7, rs.getInt(1));
       assertTrue(rs.next());
-      assertEquals(9, rs.getInt(1));
+      assertEquals(8, rs.getInt(1));
       assertFalse(rs.next());
     }
   }
